package ru.vyatsu.qr_access_api.repository

import org.springframework.jdbc.core.JdbcTemplate
import org.springframework.stereotype.Repository
import ru.vyatsu.models.QrCode
import java.time.ZoneId
import java.util.*

private const val GET_ACTUAL_QRS_BY_UNIT_ID = """
            SELECT q.start_date_time, q.end_date_time, q.door_id, q.key_code FROM qrs q
            JOIN doors d ON (d.id = q.door_id)
            JOIN oauth2_authorized_client c ON (c.client_registration_id = d.unit_id AND c.principal_name = d.principal_name)
            WHERE c.client_registration_id = ? AND q.start_date_time <= CURRENT_TIMESTAMP AND q.end_date_time >= CURRENT_TIMESTAMP
            """

@Repository
class QrRepository(private val template: JdbcTemplate) {

    fun getActualQrCodesByUnitId(unitId: String): List<QrCode> {
        val query = GET_ACTUAL_QRS_BY_UNIT_ID.trimIndent()
        return template.query({ conn ->
            val stmt = conn.prepareStatement(query)
            stmt.setString(1, unitId)
            stmt
        }, { rs, _ ->
            QrCode(
                rs.getTimestamp("start_date_time").toLocalDateTime().atZone(ZoneId.systemDefault()).toOffsetDateTime(),
                rs.getTimestamp("end_date_time").toLocalDateTime().atZone(ZoneId.systemDefault()).toOffsetDateTime(),
                UUID.fromString(rs.getString("door_id")),
                UUID.fromString(rs.getString("key_code"))
            )
        })
    }

}