package ru.vyatsu.qr_access_api.database.utils

import org.springframework.jdbc.core.JdbcTemplate
import java.time.LocalDateTime
import java.time.OffsetDateTime

private const val INSERT_CLIENT_QUERY =
    """INSERT INTO oauth2_authorized_client(client_registration_id, principal_name, access_token_type, access_token_value, access_token_issued_at, access_token_expires_at, created_at) 
       VALUES (?, ?, ?, ?, ?, ?, ?)"""

class InsertDatabaseHelper(private val template: JdbcTemplate) {
    fun insertQr(
        keyCode: String,
        doorId: String,
        startDateTime: OffsetDateTime,
        endDateTime: OffsetDateTime
    ): Int {
        return template.update("INSERT INTO qrs(key_code, door_id, start_date_time, end_date_time) VALUES (?, ?, ?, ?)") { ps ->
            ps.setString(1, keyCode)
            ps.setString(2, doorId)
            ps.setObject(3, startDateTime)
            ps.setObject(4, endDateTime)
        }
    }

    fun insertDoor(id: String, unitId: String, principalName: String = ""): Int {
        return template.update("INSERT INTO doors(id, unit_id, principal_name) VALUES (?, ?, ?)") { ps ->
            ps.setString(1, id)
            ps.setString(2, unitId)
            ps.setString(3, principalName)
        }
    }

    fun insertClient(id: String, principalName: String = ""): Int {
        return template.update(INSERT_CLIENT_QUERY) { ps ->
            ps.setString(1, id)
            ps.setString(2, principalName)
            ps.setString(3, "Bearer")
            ps.setString(4, "Tokenasfgerseawvg")
            ps.setObject(5, LocalDateTime.now())
            ps.setObject(6, LocalDateTime.now())
            ps.setObject(7, LocalDateTime.now())
        }
    }
}