package ru.vyatsu.qr_access_api.repository

import org.assertj.core.api.Assertions.assertThat
import org.junit.jupiter.api.BeforeEach
import org.junit.jupiter.api.Test
import org.springframework.transaction.annotation.Transactional
import ru.vyatsu.models.QrCode
import java.time.OffsetDateTime
import java.time.temporal.ChronoUnit
import java.util.*

@Transactional
class QrRepositoryTest : RepositoryTest() {


    @BeforeEach
    fun setUp() {

    }

    @Test
    fun checkThatQrCodesReturnedWhenGetActualQrCodesByUnitIdMappingCheck() {
        val clientId = "9c0cc999-53b1-4ff1-b13c-5cd1514fec3a"
        val doorId = "430141c2-858f-481e-93d9-2dffaf83ea2e"
        val keyCode = "e1148f54-96aa-4355-b7f8-6eb0f549f5eb"
        val startDateTime = OffsetDateTime.now().minusMinutes(30).truncatedTo(ChronoUnit.MICROS)
        val endDateTime = OffsetDateTime.now().plusMinutes(30).truncatedTo(ChronoUnit.MICROS)
        insertDatabaseHelper.insertClient(clientId)
        insertDatabaseHelper.insertDoor(doorId, clientId)
        insertDatabaseHelper.insertQr(
            keyCode,
            doorId,
            startDateTime,
            endDateTime
        )

        val codes = qrRepository.getActualQrCodesByUnitId(clientId)

        assertThat(codes).containsOnly(
            QrCode(
                startDateTime,
                endDateTime,
                UUID.fromString(doorId),
                UUID.fromString(keyCode)
            )
        )
    }

    @Test
    fun checkThatQrCodesReturnedWhenGetActualQrCodesByUnitIdForOneDoor() {
        val clientId = "9c0cc999-53b1-4ff1-b13c-5cd1514fec3a"
        val doorId = "430141c2-858f-481e-93d9-2dffaf83ea2e"
        insertDatabaseHelper.insertClient(clientId)
        insertDatabaseHelper.insertDoor(doorId, clientId)
        insertDatabaseHelper.insertQr(
            "e1148f54-96aa-4355-b7f8-6eb0f549f5eb",
            doorId,
            OffsetDateTime.now().minusMinutes(20),
            OffsetDateTime.now().plusMinutes(20)
        )
        insertDatabaseHelper.insertQr(
            "a9fc474b-1871-4257-b174-1b6e36d393a8",
            doorId,
            OffsetDateTime.now().minusMinutes(30),
            OffsetDateTime.now().plusMinutes(30)
        )
        insertDatabaseHelper.insertQr(
            "b3bfe837-de12-4c07-b39a-4296266f2a8c",
            doorId,
            OffsetDateTime.now().minusMinutes(30),
            OffsetDateTime.now().minusMinutes(20)
        )

        val codes = qrRepository.getActualQrCodesByUnitId(clientId)

        assertThat(codes).hasSize(2)
    }

    @Test
    fun checkThatQrCodesReturnedWhenGetActualQrCodesByUnitIdForFewDoors() {
        val clientId = "9c0cc999-53b1-4ff1-b13c-5cd1514fec3a"
        insertDatabaseHelper.insertClient(clientId)
        insertDatabaseHelper.insertDoor("430141c2-858f-481e-93d9-2dffaf83ea2e", clientId)
        insertDatabaseHelper.insertDoor("259411de-6d90-46a4-92fb-4028a33ad48e", clientId)
        insertDatabaseHelper.insertQr(
            "e1148f54-96aa-4355-b7f8-6eb0f549f5eb",
            "430141c2-858f-481e-93d9-2dffaf83ea2e",
            OffsetDateTime.now().minusMinutes(20),
            OffsetDateTime.now().plusMinutes(20)
        )
        insertDatabaseHelper.insertQr(
            "a9fc474b-1871-4257-b174-1b6e36d393a8",
            "259411de-6d90-46a4-92fb-4028a33ad48e",
            OffsetDateTime.now().minusMinutes(30),
            OffsetDateTime.now().plusMinutes(30)
        )
        insertDatabaseHelper.insertQr(
            "b3bfe837-de12-4c07-b39a-4296266f2a8c",
            "430141c2-858f-481e-93d9-2dffaf83ea2e",
            OffsetDateTime.now().minusMinutes(30),
            OffsetDateTime.now().minusMinutes(20)
        )
        insertDatabaseHelper.insertQr(
            "90a3de92-33ed-4752-8eff-d425778e1381",
            "259411de-6d90-46a4-92fb-4028a33ad48e",
            OffsetDateTime.now().minusMinutes(30),
            OffsetDateTime.now().minusMinutes(20)
        )

        val codes = qrRepository.getActualQrCodesByUnitId(clientId)

        assertThat(codes).hasSize(2)
    }

    @Test
    fun checkThatQrCodesEmptyWhenGetActualQrCodesByUnitIdForOneDoor() {
        val clientId = "9c0cc999-53b1-4ff1-b13c-5cd1514fec3a"
        insertDatabaseHelper.insertClient(clientId)
        insertDatabaseHelper.insertDoor("430141c2-858f-481e-93d9-2dffaf83ea2e", clientId)
        insertDatabaseHelper.insertQr(
            "e1148f54-96aa-4355-b7f8-6eb0f549f5eb",
            "430141c2-858f-481e-93d9-2dffaf83ea2e",
            OffsetDateTime.now().minusMinutes(30),
            OffsetDateTime.now().minusMinutes(20)
        )
        insertDatabaseHelper.insertQr(
            "b3bfe837-de12-4c07-b39a-4296266f2a8c",
            "430141c2-858f-481e-93d9-2dffaf83ea2e",
            OffsetDateTime.now().minusMinutes(30),
            OffsetDateTime.now().minusMinutes(20)
        )

        val codes = qrRepository.getActualQrCodesByUnitId(clientId)

        assertThat(codes).hasSize(0)
    }
}